# -*- coding: utf-8 -*-
"""optimg main"""
__copyright__ = "Copyright (c) 2025 https://github.com/davidohnee"

import os
import logging
import argparse
from tqdm import tqdm
from .optimise import optimise_image
from .dir import ensure_dir_exists, get_images_in_dir

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


def _get_args() -> argparse.Namespace:
    """get arguments"""
    parser = argparse.ArgumentParser(description="Optimise images")
    parser.add_argument("--input-dir", "-i", type=str, default="./", help="Input dir")
    parser.add_argument("--output-dir", "-o", type=str, default="./out/", help="Output dir")
    parser.add_argument(
        "--resize-mode",
        type=str,
        default=None,
        help="Type [f]it & crop or [c]ontain",
        choices=["f", "c", None],
    )
    parser.add_argument("--max-res", type=int, default=2560, help="Max resolution (px)")
    parser.add_argument("--format", "-f", type=str, default="webp", help="Output image format")
    parser.add_argument(
        "--quality",
        type=int,
        default=100,
        help="Output image quality (1-100) or compression level if lossless (1-100)",
    )
    parser.add_argument(
        "--lossless", action="store_true", help="Use lossless compression (webp only)"
    )
    return parser.parse_args()


def main() -> None:
    """main"""
    args = _get_args()

    input_dir: str = args.input_dir
    output_dir: str = args.output_dir

    ensure_dir_exists(output_dir)
    files = get_images_in_dir(input_dir)

    # max_res specified but resize_mode not specified
    if args.max_res is not None and args.resize_mode is None:
        args.resize_mode = "c"
        logger.info("Auto setting resize mode to 'c' (contain). Use --resize-mode to override.")

    if args.lossless and args.format.lower() != "webp":
        logger.warning("Lossless option is only supported for webp format. Ignoring --lossless.")

    for file in tqdm(files, desc="Optimising images"):
        optimise_image(
            os.path.join(input_dir, file),
            output_dir,
            format_ext=args.format,
            max_res=args.max_res,
            resize_mode=args.resize_mode,
            quality=args.quality,
            lossless=args.lossless,
        )


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""directory utilities for optimg"""
__copyright__ = "Copyright (c) 2025 https://github.com/davidohnee"

import os
import re


def get_images_in_dir(input_dir: str) -> list[str]:
    """get images in dir"""
    files = os.listdir(input_dir)
    matching_files = [
        file
        for file in files
        if re.match(r".*\.(jpg|jpeg|png|gif|bmp|tiff|tif|webp)$", file, re.IGNORECASE)
    ]
    return matching_files


def ensure_dir_exists(dir_path: str) -> None:
    """ensure directory exists"""
    if not os.path.isdir(dir_path):
        os.makedirs(dir_path)



# -*- coding: utf-8 -*-
"""optimisation utilities for optimg"""
__copyright__ = "Copyright (c) 2025 https://github.com/davidohnee"


import os
import logging

from PIL import Image, ImageOps

logger = logging.getLogger(__name__)


def _parse_format(format_ext: str) -> str:
    """format extension"""
    format_ext = format_ext.lower()
    if format_ext == "jpg":
        format_ext = "jpeg"
    return format_ext


def optimise_image(
    image_path: str,
    output_dir: str,
    *,
    format_ext: str = "webp",
    max_res: int | None = None,
    resize_mode: str | None = "c",
    lossless: bool = False,
    quality: int = 80,
) -> str:
    """
    Optimise an image and save it to the output directory.

    :param image_path: Path to the input image.
    :param output_dir: Directory to save the optimised image.
    :param format_ext: Output image format extension (default: "webp").
    :param max_res: Maximum resolution (px) for resizing (default: None).
    :param resize_mode: Resize mode - "f" for fit & crop, "c" for contain (default: "c").
    :param lossless: Use lossless compression (webp only) (default: False).
    :param quality: Output image quality (1-100) or compression level if lossless (1-100).
    :return: Path to the optimised image.
    """
    # Open an image
    image = Image.open(image_path)

    # Resize the image
    if resize_mode is not None and max_res is not None:
        fixed_image = ImageOps.exif_transpose(image)
        if resize_mode == "f":
            logger.debug("Resizing image %s with fit & crop to %d px", image_path, max_res)
            image = ImageOps.fit(fixed_image, (max_res, max_res))
        elif resize_mode == "c":
            logger.debug("Resizing image %s with contain to %d px", image_path, max_res)
            image = ImageOps.contain(fixed_image, (max_res, max_res))

    file_wo_ext = os.path.splitext(os.path.basename(image_path))[0]
    output = os.path.join(output_dir, file_wo_ext + "." + format_ext)

    # Save the image
    try:
        logger.debug(
            "Saving image %s as %s with format %s, quality %d, lossless %s",
            image_path,
            output,
            _parse_format(format_ext),
            quality,
            lossless,
        )
        image.save(
            output,
            _parse_format(format_ext),
            optimize=True,
            quality=quality,
            lossless=lossless,
        )
        return output
    except OSError as e:
        if "RGBA" in str(e):
            logger.debug("Converting RGBA to RGB for %s", image_path)
            # Convert RGBA to RGB if format does not support alpha channel
            image = image.convert("RGB")
            image.save(
                output,
                _parse_format(format_ext),
                optimize=True,
                quality=quality,
                lossless=lossless,
            )
            return output
        else:
            logger.error("Error saving image %s: %s", image_path, e)
            raise e
